class ParameterStyle:
    FORM = "form"
    SIMPLE = "simple"
    LABEL = "label"
    MATRIX = "matrix"
    PIPE_DELIMITED = "pipeDelimited"
    SPACE_DELIMITED = "spaceDelimited"
    DEEP_OBJECT = "deepObject"
